/*************************************************************************
 * File Name: NSColor_MiscExtensions.m
 * Version  : 0.0 alpha
 * Date     : Thu 31-Aug-1997
 *************************************************************************
 *  COPYWHAT (C) 1997 by Tomi Engel
 *
 * This notice may not be removed from this source code.
 * The use and distribution of this software is governed by the
 * terms of the MiscKit license agreement.  Refer to the license
 * document included with the MiscKit distribution for the terms.
 *                    ALL RIGHTS RESERVED
 *
 *************************************************************************
 * Notes      :
 * Bugs       :
 * Author(s)  : tsengel
 * Last update: $Date: 1997/08/31 20:02:57 $
 * History    : $Log: NSColor_MiscExtensions.m,v $
 * History    : Revision 1.0  1997/08/31 20:02:57  tsengel
 * History    : Added to the MiscKit repository
 * History    :
 *************************************************************************/ 

#import "NSColor_MiscExtensions.h"

@implementation NSColor (MiscExtensions)

/*"
This category is mainly intended to simplify storing colors in the defaults database. For this task we need a NSPPL compliant object like an NSString.
"*/

+ (NSColor *)colorWithCalibratedRgbaStringRepresentation:(NSString *)aString
/*"
Returns the NSColor instance created by #colorWithCalibratedRed:green:blue:alpha: based on the four values stored in the string.
"*/
{
    id		colorStringArray = [aString componentsSeparatedByString:@" "];

    if( [colorStringArray count] != 4 ) return nil;

    return [self colorWithCalibratedRed:[[colorStringArray objectAtIndex:0] floatValue]
                                  green:[[colorStringArray objectAtIndex:1] floatValue]
                                   blue:[[colorStringArray objectAtIndex:2] floatValue]
                                  alpha:[[colorStringArray objectAtIndex:3] floatValue]];
}

+ (NSColor *)colorWithDeviceRgbaStringRepresentation:(NSString *)aString
/*"
Returns the NSColor instance created by #colorWithDeviceRed:green:blue:alpha: based on the four values stored in the string.
"*/
{
    id		colorStringArray = [aString componentsSeparatedByString:@" "];

    if( [colorStringArray count] != 4 ) return nil;

    return [self colorWithDeviceRed:[[colorStringArray objectAtIndex:0] floatValue]
                              green:[[colorStringArray objectAtIndex:1] floatValue]
                               blue:[[colorStringArray objectAtIndex:2] floatValue]
                              alpha:[[colorStringArray objectAtIndex:3] floatValue]];
}

- (NSString *)rgbaStringRepresentation
/*"
     Returns a string which represents the four RGBA values of the current color
"*/
{
    return [NSString stringWithFormat:@"%f %f %f %f",
        [self redComponent],
        [self greenComponent],
        [self blueComponent],
        [self alphaComponent]];
}

@end
