/* TSAppManager.m created by tsengel on Thu 14-Aug-1997 */

#import "TSAppManager.h"

#import <MiscAppKit/MiscAppKit.h>

#import "TSShelfWindow.h"
#import "TSShelfItem.h"
#import "NSDictionary_MiscDefaultsSupport.h"
#import "NSColor_MiscExtensions.h"

#import <OFInfoManager.h>

@implementation TSAppManager

- (void)applicationDidFinishLaunching:(NSNotification *)notification
/*"
    Prepare the application.
"*/
{
    id		aString;

    [self registerDefaults];

    // Build the shelf window
    // For some magical reasons autosizing only works in 80% of the cases when we open the shelf window
    // as it is defined in the NIB... In 20% it simply forgets to resize its content view...which is bad.
    // Therefore we create it "by hand".

    shelfWindow = [[TSShelfWindow alloc]  initWithContentRect:NSMakeRect( 100, 100, 100, 100)
                                                    styleMask:NSBorderlessWindowMask
                                                      backing:NSBackingStoreBuffered
                                                        defer:NO];

    [shelfWindow setFrameAutosaveName:@"ShelfWindow"];
    [shelfWindow setFrameUsingName:@"ShelfWindow"];

    // Now that we have the final size and position we will insert the contentView.

    [shelfWindow setContentView:shelfContentView];

    _fixedMatrixRect = [tabMatrix frame];
    [shelfWindow setGrabBorder:_fixedMatrixRect.size.height]; 	

    aString = [[NSUserDefaults standardUserDefaults] stringForKey:@"DesktopColor"];
    [desktopColorPrefWell setColor:[NSColor colorWithCalibratedRgbaStringRepresentation:aString]];
    [self desktopColorChanged:self];

    // We will record the current Matrix rect since we will often have to resize it again.
    // After that...laod all the levels into the shelf

    [self loadAllShelfLevels];

    [shelfWindow setLevel:[[NSUserDefaults standardUserDefaults] integerForKey:@"ShelfWindowLevel"]];
    [shelfWindow orderFront:self];
}

- (void)registerDefaults
{
    NSMutableDictionary* registrationDict = [NSMutableDictionary dictionary];
    id		anArray;
    id		aDict = nil;

    // Internal defaults. Not accessible by the prefs panel

    [registrationDict setObject:@"~/Library/TheShelf" forKey:@"DefaultShelfLibraryPath"];
    [registrationDict setObject:NSHomeDirectory() forKey:@"LastOpenDirectory"];
    [registrationDict setInteger:TSShelfForegroundWindowLevel forKey:@"ShelfWindowLevel"];

    // Now specify the default shelf that we will come up with.
    // Title is the tab title, path the path to the storage...if there is no path then there should be
    // a type which is the shelf bundles name.

    anArray = [NSMutableArray new];
    aDict = [NSDictionary dictionaryWithObjectsAndKeys: @"Applications", @"Title", @"IconViewShelf", @"ShelfType", nil];
    [anArray addObject:aDict];
    aDict = [NSDictionary dictionaryWithObjectsAndKeys: @"Tray", @"Title", @"SystemTrayShelf", @"ShelfType", nil];
    [anArray addObject:aDict];
    aDict = [NSDictionary dictionaryWithObjectsAndKeys: @"Documents", @"Title", @"IconViewShelf", @"ShelfType", nil];
    [anArray addObject:aDict];
    aDict = [NSDictionary dictionaryWithObjectsAndKeys: @"Temporary", @"Title", @"IconViewShelf", @"ShelfType", nil];
    [anArray addObject:aDict];
    aDict = [NSDictionary dictionaryWithObjectsAndKeys: @"Clip Notes", @"Title", @"TextViewShelf", @"ShelfType", nil];
    [anArray addObject:aDict];
    aDict = [NSDictionary dictionaryWithObjectsAndKeys: @"Pastebaord", @"Title", @"IconViewShelf", @"ShelfType", nil];
    [anArray addObject:aDict];

    [registrationDict setInteger:0 forKey:@"SelectedTabNumber"];
    [registrationDict setObject:@"1.0 1.0 1.0 1.0" forKey:@"DesktopColor"];
   
    [registrationDict setObject:anArray forKey:@"Shelfs"];

    [[NSUserDefaults standardUserDefaults] registerDefaults:registrationDict];
}

- (void)desktopColorChanged:(id)sender
{
    [[NSUserDefaults standardUserDefaults] setObject:[[desktopColorPrefWell color] rgbaStringRepresentation]
                                              forKey:@"DesktopColor"];
    [desktopFakeView setBackgroundColor:[desktopColorPrefWell color]];
    [desktopFakeView setNeedsDisplay:YES];
}

- (BOOL)loadAllShelfLevels
{
    id		defaultShelfs = [[NSUserDefaults standardUserDefaults] objectForKey:@"Shelfs"];
    id		enumerator;
    id		nextDict;
    id		anItem;

    id		loadPath;

    int		selColumn = [[NSUserDefaults standardUserDefaults] integerForKey:@"SelectedTabNumber"];

    // First we will remove all shelf levels which are currently active

    while( [tabMatrix numberOfColumns] > 0 )
        [self removeShelfLevel:self];

    // And then lets add the default tray level

    enumerator = [defaultShelfs objectEnumerator];
    while( nextDict = [enumerator nextObject] )
    {
        loadPath = [nextDict objectForKey:@"Path"];
        if( loadPath )
            anItem = [self newItemWithPath:loadPath];
        else
            anItem = [self newItemWithName:[nextDict objectForKey:@"ShelfType"]];

        if( anItem )
        {
            [anItem setTitle:[nextDict objectForKey:@"Title"]];
            [self addShelfLevelItem:anItem];
        }
    }

    // Finally try to preselect the right shelf level...
    // Select the tab..ugly !!!

    if( selColumn < [tabMatrix numberOfColumns] )
        [tabMatrix selectCellAtRow:0 column:selColumn];
    [tabMatrix sendAction];

    return YES;
}

- (void)applicationWillTerminate:(NSNotification *)aNotification
{
    // Ensure that all shelf levels are saved. This also does a sync on the user defaults..so everything should be secure by now

    NSLog( @"TheShelf....terminating and saving...");
    [self saveShelfs:self];
}

- (BOOL)application:(NSApplication *)application openFile:(NSString *)filename
{
    // <<HACk>> We should check if there are times when we have to return no here !

    return [self _openFile:filename];
}

- (void)open:(id)sender
    /*"
    Here comes mehtod docu...right here
    "*/
{
    NSArray* 		files;
    NSArray* 		acceptedFileTypes = [NSArray arrayWithObjects:@"aShelf", nil];
    NSOpenPanel*	openPanel = [NSOpenPanel openPanel];
    int				numberOfFiles;
    int				index = 0;

    [openPanel setAllowsMultipleSelection:YES];
    [openPanel setTreatsFilePackagesAsDirectories:NO];
    [openPanel setTitle:@"Open Shelf"];

    if( [openPanel runModalForDirectory:[[NSUserDefaults standardUserDefaults] objectForKey:@"LastOpenDirectory"]
                                   file:nil
                                  types:acceptedFileTypes] )
    {
        // This is overkill since ther are no multiplefiles right now... but we'll just leave the code in here..doesn't hurt.

        files = [openPanel filenames];
        numberOfFiles = [files count];
        while( index < numberOfFiles )
        {
            [self _openFile:[files objectAtIndex:index]];
            index++;
        }

        // Now remember where the user loaded the last file from.
        // We will by default look at this place first once we have to load a file again.

        [[NSUserDefaults standardUserDefaults] setObject:[openPanel directory]
                                                  forKey:@"LastOpenDirectory"];
        // Select the tab..ugly !!!

        [tabMatrix selectCellAtRow:0 column:( [tabMatrix numberOfColumns] - 1 )];
        [tabMatrix sendAction];
    }
}

- (BOOL)_openFile:(NSString *)filename
{
    id		anItem;

    anItem = [self newItemWithPath:filename];
    if( anItem )
    {
        [self addShelfLevelItem:anItem];
        return YES;
    }
   return NO;
}


- (void)addShelfLevel:(id)sender
{
    [self addIconShelfLevel:self];
}

- (void)addIconShelfLevel:(id)sender
{
    [self addShelfLevelItem:[self newItemWithName:@"IconViewShelf"]];

    // Select the tab..ugly !!!

    [tabMatrix selectCellAtRow:0 column:( [tabMatrix numberOfColumns] - 1 )];
    [tabMatrix sendAction];
}

- (void)addTextShelfLevel:(id)sender
{
    [self addShelfLevelItem:[self newItemWithName:@"TextViewShelf"]];

    // Select the tab..ugly !!!

    [tabMatrix selectCellAtRow:0 column:( [tabMatrix numberOfColumns] - 1 )];
    [tabMatrix sendAction];
}

- (void)addTrayShelfLevel:(id)sender
{
    [self addShelfLevelItem:[self newItemWithName:@"SystemTrayShelf"]];

    // Select the tab..ugly !!!

    [tabMatrix selectCellAtRow:0 column:( [tabMatrix numberOfColumns] - 1 )];
    [tabMatrix sendAction];
}

- (id)newItemWithName:(NSString *)bundleName
{
    id		path;
    id		newItem;
    id		ourBundle;

    path = [[NSBundle mainBundle] pathForResource:bundleName ofType:@"bundle"];
    ourBundle = [NSBundle bundleWithPath:path];
    newItem = [[ourBundle principalClass] new];

    return newItem;
}

- (id)newItemWithPath:(NSString *)shelfPath
{
    id		path;
    id		newItem = nil;
    id		ourBundle;
    id		shelfType;
    id		plistDict = [NSDictionary dictionaryWithContentsOfFile:[shelfPath stringByAppendingPathComponent:@"Info.plist"]];

    // NSLog( @"Load shelf from path %@", plistDict );

    shelfType = [plistDict objectForKey:@"ShelfType"];
    if( shelfType )
    {
        path = [[NSBundle mainBundle] pathForResource:shelfType ofType:@"bundle"];
        ourBundle = [NSBundle bundleWithPath:path];
        newItem = [[[ourBundle principalClass] alloc] initWithContentsOfFile:shelfPath];
    }
    return newItem;
}

- (void)addShelfLevelItem:(id)newItem
{
    int		newColumn;
    id		newCell;
    id		title;

    [tabMatrix addColumn];
    newColumn = [tabMatrix numberOfColumns] - 1;

    // Now finally creat the new shelf item and select the new tab that we just added.

    newCell = [tabMatrix cellAtRow:0 column:newColumn];

    // Title is always a string...in the worst case it is the empty string.

    title = [newItem title];
    if( [title isEqual:@""] )
    {
        title = [NSString stringWithFormat:@"Shelf Nr.:%i", newColumn + 1];
        [newItem setTitle:title];
    }
    [newCell setTitle:title];
    [newCell setAllowsEditingTextAttributes:YES];

    [newItem setIdentifier:newCell];
    [swapView addSwapViewItem:newItem];

    [tabMatrix setFrame:_fixedMatrixRect];
    [tabMatrix setNeedsDisplay:YES];
}

- (void)shiftShelfTabLeft:(id)sender
{
    id		affectedCell;
    id		selectedCell;
    int		oldColumn = [tabMatrix selectedColumn];
    int		newColumn = oldColumn - 1;

    if( oldColumn > 0 )
    {
        affectedCell = [[tabMatrix cellAtRow:0 column:newColumn] retain];
        selectedCell = [[tabMatrix cellAtRow:0 column:oldColumn] retain];

        // Now swap them and reselect the new position...

        [tabMatrix putCell:selectedCell atRow:0 column:newColumn];
        [tabMatrix putCell:affectedCell atRow:0 column:oldColumn];

        [tabMatrix selectCellAtRow:0 column:oldColumn];
        [tabMatrix selectCellAtRow:0 column:newColumn];
        [tabMatrix setNeedsDisplay:YES];

        [affectedCell release];
        [selectedCell release];
    }
}

- (void)shiftShelfTabRight:(id)sender
{
    id		affectedCell;
    id		selectedCell;
    int		oldColumn = [tabMatrix selectedColumn];
    int		newColumn = oldColumn + 1;

    if( newColumn < [tabMatrix numberOfColumns] )
    {
        affectedCell = [[tabMatrix cellAtRow:0 column:newColumn] retain];
        selectedCell = [[tabMatrix cellAtRow:0 column:oldColumn] retain];

        // Now swap them and reselect the new position...

        [tabMatrix putCell:selectedCell atRow:0 column:newColumn];
        [tabMatrix putCell:affectedCell atRow:0 column:oldColumn];

        [tabMatrix selectCellAtRow:0 column:newColumn];
        [tabMatrix setNeedsDisplay:YES];

        [affectedCell release];
        [selectedCell release];
    }
}

- (void)renameShelfLevel:(id)sender
{
    id		selectedCell = [tabMatrix selectedCell];
    id		newTitle = [renamingText stringValue];

    // Ok..this could be a lot simpler etc..pp..

    if( newTitle && ![newTitle isEqual:@""] )
    {
        [selectedCell setTitle:[renamingText stringValue]];
        [[swapView swapViewItemWithIdentifier:selectedCell] setTitle:[renamingText stringValue]];
        [[sender window] performClose:self];
    }
    else
    {
        // Maybe we should bring up an alert... or ping and not close the window or that like ... ??
    }
    [renamingText setStringValue:@""];
}

- (void)removeShelfLevel:(id)sender
/*"
    Removes the currently selected shelf level.
"*/
{
    id		selectedCell;
    id		affectedItem;

    // Before we really remove the tab from the matrix we will remove the shelf item which belongs to it

    selectedCell = [tabMatrix selectedCell];
    if( !selectedCell ) return;

    // Now its time to fix the swapView as well
    // and we will save teh shelf level since if someone was too fast in removing it..he can safely reopen it again

    affectedItem = [swapView swapViewItemWithIdentifier:selectedCell];
    [affectedItem save];
    [swapView removeSwapViewItem:affectedItem];

    [tabMatrix removeColumn:[tabMatrix selectedColumn]];
    [tabMatrix setFrame:_fixedMatrixRect];
    [tabMatrix setNeedsDisplay:YES];
}

- (void)dragCurrentShelfLevel:(id)sender
{
}

- (void)saveShelfs:(id)sender
{	
    // Trigger a save for all shelf items we know

    id		anItem;
    id		aCell;
    int		column;

    id		aDict;
    id		itemArray = [NSMutableArray new];

    // Now we also should put the current shelf info into our defaults database !

    for( column=0; column<[tabMatrix numberOfColumns]; column++ )
    {
        // Now get each shelf item and save it if its content has been changed.

        aCell = [tabMatrix cellAtRow:0 column:column];
        anItem = [swapView swapViewItemWithIdentifier:aCell];
        if( [anItem contentDidChange] ) [anItem save];

        aDict = [NSMutableDictionary new];
        [aDict setObject:[anItem title] forKey:@"Title"];
        [aDict setObject:[anItem path] forKey:@"Path"];

        [itemArray addObject:aDict];
    }
    // NSLog( @"Writing defaults array %@", itemArray );

    [[NSUserDefaults standardUserDefaults] setObject:itemArray forKey:@"Shelfs"];
    [[NSUserDefaults standardUserDefaults] setInteger:[tabMatrix selectedColumn]
                                               forKey:@"SelectedTabNumber"];

    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (void)showInfoPanel:sender
/*
   Display the info panel and make it the key window.
*/
{
    // Ok...we are not really using an info "panel"...but its a
    // class that will do the trick.

    if( !infoPanel ) infoPanel = [OFInfoManager new];
    [infoPanel makeKeyAndOrderFront:self];
}

@end
